﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Common.Log;
using EntornoWeb.Common;
using NHibernate_SGF.Entities;
using NHibernate_SGF.Services;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador del mantenimiento de colectivos y sus empleados asignados
    /// </summary>
    [HandleError]
    public class MColectivosController : SGFController
    {
        #region Colectivos
        /// <summary>
        /// Capturador que obtiene los colectivos que aparecerán en la tabla
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <returns>Colección con los colectivos solicitados</returns>
        [HandleError(ExceptionType = typeof(NullReferenceException), View = "Error")]
        public ActionResult ObtenerColectivos(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Colectivo> datos;
            Colectivo colectivo;
            if (_search)
            {
                switch (searchField)
                {
                    case "Nombre":
                        colectivo = new Colectivo();
                        colectivo.Nombre = searchString;
                        datos = ColectivoService.ObtenerLista(colectivo);
                        break;
                    case "Descripcion":
                        colectivo = new Colectivo();
                        colectivo.Descripcion = searchString;
                        datos = ColectivoService.ObtenerLista(colectivo);
                        break;
                    default:
                        datos = ColectivoService.ObtenerLista();
                        break;
                }
            }
            else
            {
                datos = ColectivoService.ObtenerLista();
            }

            Func<Colectivo, IComparable> funcOrden = ColectivoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               a.Nombre,
                               a.Descripcion,
                               a.EsOcupacion.ToString()
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar los colectivos
        /// </summary>
        /// <param name="colectivo">Datos del colectivo introducido por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarColectivo(Colectivo colectivo, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");

            switch (formCollection["oper"])
            {
                case "add":
                    colectivo.EsOcupacion = false;
                    ColectivoService.Insertar(colectivo);
                    break;
                case "edit":
                    colectivo.Idint = Int32.Parse(formCollection["id"]);
                    ColectivoService.Actualizar(colectivo);
                    break;
                case "del":
                    colectivo = ColectivoService.Obtener(Int32.Parse(formCollection["id"]));
                    colectivo.FechaBaja = DateTime.Now;
                    ColectivoService.Actualizar(colectivo);
                    break;
            }
            LogFactory.Escribir().Info("Fin");
            return Json("OK");  
        }

        /// <summary>
        /// Validación para impedir la eliminación de una ocupación
        /// </summary>
        /// <param name="colectivoIdint">Identificador del colectivo</param>
        public void ColectivoValidarDel(int colectivoIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            Colectivo colectivo = ColectivoService.Obtener(colectivoIdint);
            if (colectivo.EsOcupacion)
            {
                ShowError(MensajeUsuario.STR_INFO_ELIMINAR_OCUPACION);
            }
            LogFactory.Escribir().Info("Fin");
        }
        #endregion

        #region Empleados
        /// <summary>
        /// Capturador que obtiene los empleados asignados al colectivo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="colectivoActual">Identificador del colectivo</param>
        /// <returns>Colección de empleados asignados al colectivo</returns>
        public ActionResult ObtenerEmpleadosAsignados(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int colectivoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Empleado> datos;
            if (colectivoActual != 0)
            {
                Colectivo colectivo = new Colectivo();
                colectivo.Idint = colectivoActual;

                Empleado empleado = new Empleado();

                if (_search)
                {
                    switch (searchField)
                    {
                        case "DNI":
                            empleado.DNI = searchString;
                            datos = EmpleadoService.ObtenerLista(empleado, colectivo);
                            break;
                        case "Nombre":
                            empleado.Nombre = searchString;
                            datos = EmpleadoService.ObtenerLista(empleado, colectivo);
                            break;
                        case "Apellidos":
                            empleado.Apellidos = searchString;
                            datos = EmpleadoService.ObtenerLista(empleado, colectivo);
                            break;
                        default:
                            datos = EmpleadoService.ObtenerLista(empleado, colectivo);
                            break;
                    }
                }
                else
                {
                    datos = EmpleadoService.ObtenerLista(empleado, colectivo);
                }
            }
            else
            {
                datos = new List<Empleado>();
            }

            Func<Empleado, IComparable> funcOrden = EmpleadoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               colectivoActual.ToString(),
                               a.DNI,
                               a.Nombre,
                               a.Apellidos,
                               a.Apellidos + ", " + a.Nombre,
                               EmpleadoColectivoService.Obtener(a.Idint, colectivoActual).FechaAsignacion.ToShortDateString(),
                               (a.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador que obtiene los empleados no asignados al colectivo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="colectivoActual">Identificador del colectivo</param>
        /// <returns>Colección de empleados no asignados al colectivo</returns>
        public ActionResult ObtenerEmpleadosAsignadosDistintoColectivo(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int colectivoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Empleado> datos;
            if (colectivoActual != 0)
            {
                Colectivo colectivo = new Colectivo();
                colectivo.Idint = colectivoActual;

                Empleado empleado = new Empleado();

                if (_search)
                {
                    switch (searchField)
                    {
                        case "DNI":
                            empleado.DNI = searchString;
                            datos = EmpleadoService.ObtenerListaDistintoColectivo(empleado, colectivo);
                            break;
                        case "Nombre":
                            empleado.Nombre = searchString;
                            datos = EmpleadoService.ObtenerListaDistintoColectivo(empleado, colectivo);
                            break;
                        case "Apellidos":
                            empleado.Apellidos = searchString;
                            datos = EmpleadoService.ObtenerListaDistintoColectivo(empleado, colectivo);
                            break;
                        default:
                            datos = EmpleadoService.ObtenerListaDistintoColectivo(empleado, colectivo);
                            break;
                    }
                }
                else
                {
                    datos = EmpleadoService.ObtenerListaDistintoColectivo(empleado, colectivo);
                }
            }
            else
            {
                datos = new List<Empleado>();
            }

            Func<Empleado, IComparable> funcOrden = EmpleadoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               null,
                               a.DNI,
                               a.Nombre,
                               a.Apellidos,
                               a.Apellidos + ", " + a.Nombre,
                               (a.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Desasignación del empleado al colectivo
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="colectivoIdint">Identificador del colectivo</param>
        public void DesasignarEmpleado(int empleadoIdint, int colectivoIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            EmpleadoColectivo empleadoColectivo = EmpleadoColectivoService.Obtener(empleadoIdint, colectivoIdint);

            if (!empleadoColectivo.Colectivo.EsOcupacion)
            {
                EmpleadoService.Eliminar(empleadoColectivo);
            }
            else
            {
                ShowError(MensajeUsuario.STR_INFO_ASIGNAR_OCUPACION);
            }
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Asignación del empleado al colectivo
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="colectivoIdint">Identificador del colectivo</param>
        [HttpPost]
        public void AsignarEmpleado(int empleadoIdint, int colectivoIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            Colectivo colectivo = ColectivoService.Obtener(colectivoIdint);
            Empleado empleado = EmpleadoService.Obtener(empleadoIdint);
            EmpleadoColectivo empleadoOcupacion = EmpleadoColectivoService.ObtenerOcupacion(empleadoIdint);     

            EmpleadoColectivo empleadoColectivo = new EmpleadoColectivo();
            empleadoColectivo.Colectivo = colectivo;
            empleadoColectivo.Empleado = empleado;
            if (empleadoOcupacion != null && empleadoOcupacion.FechaAsignacion != DateTime.MinValue)
            {
                empleadoColectivo.FechaAsignacion = empleadoOcupacion.FechaAsignacion;
            }
            else
            {
                empleadoColectivo.FechaAsignacion = empleado.FechaIngreso;
            }

            EmpleadoColectivoService.Insertar(empleadoColectivo);

            if (colectivo.EsOcupacion && empleadoOcupacion != null)
            {
                EmpleadoColectivoService.Eliminar(empleadoOcupacion);
            }

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Modificación de la asignación del empleado al colectivo
        /// </summary>
        /// <param name="formCollection">Colección de datos indicados por el usuario</param>
        public void EditarAsignacion(FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            int intEmpleado = int.Parse(formCollection["id"]);
            int colectivoIdint = int.Parse(formCollection["ColectivoIdint"]);

            EmpleadoColectivo empleadoColectivo = EmpleadoColectivoService.Obtener(intEmpleado, colectivoIdint);
            empleadoColectivo.FechaAsignacion = DateTime.Parse(formCollection["Asignacion"]);

            EmpleadoColectivoService.Actualizar(empleadoColectivo);
            LogFactory.Escribir().Info("Fin");
        }
        #endregion
    }
}
