﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Common.Log;
using EntornoWeb.Common;
using NHibernate_SGF.Entities;
using NHibernate_SGF.Services;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador del mantenimiento de cursos, versiones y planificaciones
    /// </summary>
    [HandleError]
    public class MCursosController : SGFController
    {
        private const string STR_FICHERO_TEMARIO = "Temario";
        private const string STR_FICHERO_FICHA = "Ficha";

        #region Cursos
        /// <summary>
        /// Capturador que obtiene los cursos que aparecerán en la tabla
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <returns>Colección con los cursos solicitados</returns>
        public ActionResult ObtenerCursos(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Curso> datos;
            Curso curso;
            if (_search)
            {
                switch (searchField)
                {
                    case "Nombre":
                        curso = new Curso();
                        curso.Nombre = searchString;
                        datos = CursoService.ObtenerLista(curso);
                        break;
                    case "Descripcion":
                        curso = new Curso();
                        curso.Descripcion = searchString;
                        datos = CursoService.ObtenerLista(curso);
                        break;
                    case "Tipo":
                        curso = new Curso();
                        curso.Tipo = searchString;
                        datos = CursoService.ObtenerLista(curso);
                        break;
                    case "IncorporacionMeses":
                        curso = new Curso();
                        curso.IncorporacionMeses = int.Parse(searchString);
                        datos = CursoService.ObtenerLista(curso);
                        break;
                    case "PeriodicidadMeses":
                        curso = new Curso();
                        curso.PeriodicidadMeses = int.Parse(searchString);
                        datos = CursoService.ObtenerLista(curso);
                        break;
                    default:
                        datos = CursoService.ObtenerLista();
                        break;
                }
            }
            else
            {
                datos = CursoService.ObtenerLista();
            }

            Func<Curso, IComparable> funcOrden = CursoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               a.Nombre,
                               a.Descripcion,
                               a.ToString(),
                               a.IncorporacionMeses.ToString(),
                               a.PeriodicidadMeses.ToString(),
                               a.RepetirConNuevaVersion.ToString()
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar los cursos
        /// </summary>
        /// <param name="cursoGrid">Datos del curso introducido por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarCurso(Curso cursoGrid, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");

            Curso curso;     
            switch (formCollection["oper"])
            {
                case "add":
                    curso = CambiarTipo(cursoGrid, formCollection["Tipo"], true);
                    curso.Nombre = cursoGrid.Nombre;
                    curso.Descripcion = cursoGrid.Descripcion;
                    curso.IncorporacionMeses = cursoGrid.IncorporacionMeses;
                    curso.PeriodicidadMeses = cursoGrid.PeriodicidadMeses;
                    curso.RepetirConNuevaVersion = Boolean.Parse(formCollection["RepetirConNuevaVersion"]);
                    curso.FechaAlta = DateTime.Now;
                    curso.FechaModi = DateTime.Now;
                    CursoService.Insertar(curso);
                    break;
                case "edit":
                    curso = CursoService.Obtener(Int32.Parse(formCollection["id"]));
                    curso = CambiarTipo(curso, formCollection["Tipo"], false);
                    curso.Nombre = cursoGrid.Nombre;
                    curso.Descripcion = cursoGrid.Descripcion;
                    curso.IncorporacionMeses = cursoGrid.IncorporacionMeses;
                    curso.PeriodicidadMeses = cursoGrid.PeriodicidadMeses;
                    curso.RepetirConNuevaVersion = Boolean.Parse(formCollection["RepetirConNuevaVersion"]);
                    curso.FechaModi = DateTime.Now;
                    CursoService.Actualizar(curso);
                    break;
                case "del":
                    curso = CursoService.Obtener(Int32.Parse(formCollection["id"]));
                    curso.FechaModi = DateTime.Now;
                    curso.FechaBaja = DateTime.Now;
                    CursoService.Actualizar(curso);
                    break;
            }

            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }

        /// <summary>
        /// Modifica el tipo del curso
        /// </summary>
        /// <param name="curso">Curso que debe cambiar de tipo</param>
        /// <param name="strTipo">Nuevo tipo del curso (1=Central/2=Local/3=Otros)</param>
        /// <param name="blnInicializar">Se debe crear una nueva instancia</param>
        /// <returns>Curso con tipología modificada</returns>
        private Curso CambiarTipo(Curso curso, String strTipo, bool blnInicializar)
        {
            switch (strTipo)
            {
                case "1":
                    if (blnInicializar) return new CursoCentral();
                    curso.Tipo = Curso.STR_IDINT_CENTRAL;
                    break;
                case "2":
                    if (blnInicializar) return new CursoLocal();
                    curso.Tipo = Curso.STR_IDINT_LOCAL;
                    break;
                case "3":
                    if (blnInicializar) return new CursoOtros();
                    curso.Tipo = Curso.STR_IDINT_OTROS;
                    break;
                default:
                    if (blnInicializar) return new CursoCentral();
                    curso.Tipo = Curso.STR_IDINT_CENTRAL;
                    break;
            }
            return curso;
        }
        #endregion

        #region Versiones
        /// <summary>
        /// Capturador que obtiene las versiones asignadas al curso indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="cursoActual">Identificador del curso</param>
        /// <returns>Colección de versiones asignadas al curso</returns>
        public ActionResult ObtenerVersiones(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int cursoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Versio> datos;
            Versio version = new Versio();

            if (cursoActual != 0)
            {
                version.Curso = CursoService.Obtener(cursoActual);

                if (_search)
                {
                    switch (searchField)
                    {
                        case "Nombre":
                            version.Nombre = searchString;
                            datos = VersionService.ObtenerLista(version);
                            break;
                        default:
                            datos = VersionService.ObtenerLista(version);
                            break;
                    }
                }
                else
                {
                    datos = VersionService.ObtenerLista(version);
                }
            }
            else
            {
                datos = new List<Versio>();
            }

            Func<Versio, IComparable> funcOrden = VersionService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               cursoActual.ToString(),
                               a.Nombre,
                               a.ToString(),
                               (a.FechaBaja == DateTime.MinValue).ToString(),
                               (a.Temario!=null ? "true": "false"),
                               (a.Temario!=null ? "true": "false"),
                               (a.Temario!=null ? "true": "false"),
                               (a.Ficha!=null ? "true": "false"),
                               (a.Ficha!=null ? "true": "false"),
                               (a.Ficha!=null ? "true": "false")
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }
        
        /// <summary>
        /// Capturador para añadir, modificar y eliminar las versiones
        /// </summary>
        /// <param name="versionGrid">Datos de la versión introducida por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarVersion(Versio versionGrid, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");

            int intCursoActual = 0;
            try
            {
                intCursoActual = Int32.Parse(formCollection["CursoIdint"]);
            }
            catch { }

            Versio version;
            switch (formCollection["oper"])
            {
                case "add":
                    version = CambiarTipo(versionGrid, formCollection["TipoVersion"], true);
                    version.Nombre = versionGrid.Nombre;
                    version.FechaAlta = DateTime.Now;
                    version.FechaModi = DateTime.Now;
                    version.Curso = CursoService.Obtener(intCursoActual);
                    if (formCollection["Activa"].Equals("False"))
                    {
                        version.FechaBaja = DateTime.Now;
                    }
                    VersionService.Insertar(version);
                    if (version.FechaBaja == DateTime.MinValue)
                    {
                        VersionService.BajaVersionesRestantes(version);
                    }
                    break;
                case "edit":
                    version = VersionService.Obtener(Int32.Parse(formCollection["id"]));
                    version.Nombre = versionGrid.Nombre;
                    version.FechaModi = DateTime.Now;
                    if (version.FechaBaja == DateTime.MinValue && formCollection["Activa"].Equals("False"))
                    {
                        version.FechaBaja = DateTime.Now;
                    }
                    else if (version.FechaBaja != DateTime.MinValue && formCollection["Activa"].Equals("True"))
                    {
                        version.FechaBaja = DateTime.MinValue;
                    }
                    version.Curso = CursoService.Obtener(intCursoActual);
                    VersionService.Actualizar(version);
                    if (version.FechaBaja == DateTime.MinValue)
                    {
                        VersionService.BajaVersionesRestantes(version);
                    }
                    break;
                case "del":
                    version = VersionService.Obtener(Int32.Parse(formCollection["id"]));
                    intCursoActual = version.Curso.Idint;
                    if (version.FechaBaja == DateTime.MinValue) VersionService.DarDeBaja(version);
                    if (version.Ediciones.Count == 0) VersionService.Eliminar(version);
                    break;
            }

            if (VersionService.ObtenerActiva(intCursoActual) == null)
            {
                VersionService.ActivarUltima(intCursoActual);
            }

            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }

        /// <summary>
        /// Modifica el tipo de la versión
        /// </summary>
        /// <param name="version">Versión que debe cambiar de tipo</param>
        /// <param name="strTipo">Nuevo tipo de la versión (1=Online/2=ADistancia/3=Presencial)</param>
        /// <param name="blnInicializar">Se debe crear una nueva instancia</param>
        /// <returns>Versión con tipología modificada</returns>
        private Versio CambiarTipo(Versio version, String strTipo, bool blnInicializar)
        {
            switch (strTipo)
            {
                case "1":
                    if (blnInicializar) return new VersionOnline();
                    version.Tipo = Versio.STR_IDINT_ONLINE;
                    break;
                case "2":
                    if (blnInicializar) return new VersionDistancia();
                    version.Tipo = Versio.STR_IDINT_DISTANCIA;
                    break;
                case "3":
                    if (blnInicializar) return new VersionPresencial();
                    version.Tipo = Versio.STR_IDINT_PRESENCIAL;
                    break;
                default:
                    if (blnInicializar) return new VersionOnline();
                    version.Tipo = Versio.STR_IDINT_ONLINE;
                    break;
            }
            return version;
        }

        /// <summary>
        /// Capturador que almacena el temario o la ficha asociada a la versión
        /// </summary>
        /// <param name="FileData">Contenedor con los datos del fichero</param>
        /// <param name="id">Identificador de la versión</param>
        /// <param name="tipo">Tipo de fichero (Temario/Ficha)</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        public string Upload(HttpPostedFileBase FileData, int id, string tipo)
        {
            LogFactory.Escribir().Info("Inicio");

            if (Request.Files.Count > 0)
            {
                Versio version = VersionService.Obtener(id);
                byte[] buffer = new byte[Request.Files[0].ContentLength];
                Request.Files[0].InputStream.Read(buffer, 0, Request.Files[0].ContentLength);
                switch (tipo)
                {
                    case STR_FICHERO_TEMARIO:
                        version.TemarioFilename = System.IO.Path.GetFileName(Request.Files[0].FileName);
                        version.Temario = buffer;
                        break;
                    case STR_FICHERO_FICHA:
                        version.FichaFilename = System.IO.Path.GetFileName(Request.Files[0].FileName);
                        version.Ficha = buffer;
                        break;
                }
                VersionService.Actualizar(version);
            }

            LogFactory.Escribir().Info("Fin");
            return "Upload OK";
        }

        /// <summary>
        /// Capturador para descargar el temario o la ficha asociada a la versión
        /// </summary>
        /// <param name="id">Identificador de la versión</param>
        /// <param name="tipo">Tipo de fichero (Temario/Ficha)</param>
        /// <returns>Temario o ficha de la versión</returns>
        public ActionResult Firmas(string id, string tipo)
        {
            LogFactory.Escribir().Info("Inicio");
            Versio version = VersionService.Obtener(Int32.Parse(id));
            LogFactory.Escribir().Info("Fin: " + version.Temario == null);
            switch (tipo)
            {
                case STR_FICHERO_TEMARIO:
                    if (version.Temario == null) return View();
                    return File(version.Temario, "application/octet-stream", Utilities.NormalizarCadena(version.TemarioFilename));
                case STR_FICHERO_FICHA:
                    if (version.Ficha == null) return View();
                    return File(version.Ficha, "application/octet-stream", Utilities.NormalizarCadena(version.FichaFilename));
                default:
                    return View();
            }
        }

        /// <summary>
        /// Capturador para eliminar el temario o la ficha asociada a la versión
        /// </summary>
        /// <param name="versionActual">Identificador de la versión</param>
        /// <param name="tipo">Tipo de fichero (Temario/Ficha)</param>
        public void FicheroEliminar(int versionActual, string tipo)
        {
            LogFactory.Escribir().Info("Inicio");
            
            Versio version = VersionService.Obtener(versionActual);
            switch (tipo)
            {
                case STR_FICHERO_TEMARIO:
                    version.TemarioFilename = null;
                    version.Temario = null;
                    break;
                case STR_FICHERO_FICHA:
                    version.FichaFilename = null;
                    version.Ficha = null;
                    break;
            }

            VersionService.Actualizar(version);

            LogFactory.Escribir().Info("Fin");
        }
        #endregion

        #region Planificaciones
        /// <summary>
        /// Capturador que obtiene las planificaciones asignadas al curso indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="cursoActual">Identificador del curso</param>
        /// <returns>Colección de planificaciones asignadas al curso</returns>
        public ActionResult ObtenerPlanificacionesAsignadas(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int cursoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Planificacion> datos;
            Planificacion planificacion = new Planificacion();

            if (cursoActual != 0)
            {
                planificacion.Curso = CursoService.Obtener(cursoActual);
                if (_search)
                {
                    switch (searchField)
                    {
                        case "Nombre":
                            planificacion.Nombre = searchString;
                            datos = PlanificacionService.ObtenerLista(planificacion);
                            break;
                        case "Descripcion":
                            planificacion.Descripcion = searchString;
                            datos = PlanificacionService.ObtenerLista(planificacion);
                            break;
                        default:
                            datos = PlanificacionService.ObtenerLista(planificacion);
                            break;
                    }
                }
                else
                {
                    datos = PlanificacionService.ObtenerLista(planificacion);
                }
            }
            else
            {
                datos = new List<Planificacion>();
            }

            Func<Planificacion, IComparable> funcOrden = PlanificacionService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               cursoActual.ToString(),
                               a.Nombre,
                               a.FechaAlta.ToShortDateString(),
                               a.Descripcion
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar las planificaciones
        /// </summary>
        /// <param name="planificacion">Datos de la planificación introducida por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarPlanificacion(Planificacion planificacion, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            
            switch (formCollection["oper"])
            {
                case "add":
                    planificacion.FechaAlta = DateTime.Parse(formCollection["Fecha"]);
                    planificacion.Curso = CursoService.Obtener(Int32.Parse(formCollection["CursoIdint"]));
                    PlanificacionService.Insertar(planificacion);
                    break;
                case "edit":
                    planificacion.Idint = Int32.Parse(formCollection["id"]);
                    planificacion.FechaAlta = DateTime.Parse(formCollection["Fecha"]);
                    planificacion.Curso = CursoService.Obtener(Int32.Parse(formCollection["CursoIdint"]));
                    PlanificacionService.Actualizar(planificacion);
                    break;
                case "del":
                    planificacion = PlanificacionService.Obtener(Int32.Parse(formCollection["id"]));
                    PlanificacionService.Eliminar(planificacion);
                    break;
            }
            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }
        #endregion
    }
}
