﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using Common.Log;
using EntornoWeb.Common;
using NHibernate_SGF.Entities;
using NHibernate_SGF.Services;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador del mantenimiento de ediciones, sesiones, convocatorias y asistencias
    /// </summary>
    [HandleError]
    public class MEdicionesController : SGFController
    {

        #region Ediciones
        /// <summary>
        /// Capturador que obtiene las ediciones que aparecerán en la tabla
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <returns>Colección con las ediciones solicitadas</returns>
        public ActionResult ObtenerEdiciones(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<EdicionSinFichero> datos;
            Edicion edicion;
            if (_search)
            {
                DateTime dtFecha;
                switch (searchField)
                {
                    case "CursoVersion":
                        edicion = new Edicion();
                        edicion.Version = new Versio();
                        edicion.Version.Curso = new Curso();
                        edicion.Version.Curso.Nombre = searchString;
                        datos = EdicionService.ObtenerListaSinFichero(edicion);
                        break;
                    case "FechaInicio":
                        edicion = new Edicion();
                        dtFecha = new DateTime();
                        if (DateTime.TryParse(searchString, out dtFecha)) edicion.FechaInicio = dtFecha;
                        datos = EdicionService.ObtenerListaSinFichero(edicion);
                        break;
                    case "FechaFin":
                        edicion = new Edicion();
                        dtFecha = new DateTime();
                        if (DateTime.TryParse(searchString, out dtFecha)) edicion.FechaFin = dtFecha;
                        datos = EdicionService.ObtenerListaSinFichero(edicion);
                        break;
                    case "SesionesDias":
                        edicion = new Edicion();
                        edicion.SesionesDias = int.Parse(searchString);
                        datos = EdicionService.ObtenerListaSinFichero(edicion);
                        break;
                    case "DuracionMinutos":
                        edicion = new Edicion();
                        edicion.DuracionMinutos = int.Parse(searchString);
                        datos = EdicionService.ObtenerListaSinFichero(edicion);
                        break;
                    default:
                        datos = EdicionService.ObtenerListaSinFichero();
                        break;
                }
            }
            else
            {
                datos = EdicionService.ObtenerListaSinFichero();
            }

            Func<EdicionSinFichero, IComparable> funcOrden = EdicionService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               a.Version.Curso.Nombre,
                               a.Version.Nombre,
                               a.Version.Curso.Nombre + " - " + a.Version.Nombre,
                               a.FechaInicio.ToShortDateString(),
                               a.FechaFin.ToShortDateString(),
                               a.SesionesDias.ToString(),
                               a.DuracionMinutos.ToString(),
                               (a.HojaFirmasFilename!=null ? "true": "false"),
                               (a.HojaFirmasFilename!=null ? "true": "false"),
                               (a.HojaFirmasFilename!=null ? "true": "false")
                           }
                       }
            };

            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar las ediciones
        /// </summary>
        /// <param name="edicionGrid">Datos de la edición introducida por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarEdicion(Edicion edicionGrid, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");

            if (formCollection["CursoVersion"] != null)
            {
                edicionGrid.Version = VersionService.Obtener(Int32.Parse(formCollection["CursoVersion"]));
            }
            edicionGrid.DuracionMinutos = edicionGrid.DuracionMinutos * 60;
            
            switch (formCollection["oper"])
            {
                case "add":
                    Sesion sesion;
                    if (!edicionGrid.Version.EqualTipo(VersionesTipo.Presencial))
                    {
                        edicionGrid.SesionesDias = 1;
                        sesion = new Sesion();
                        sesion.Numero = 1;
                        sesion.FechaInicio = edicionGrid.FechaInicio;
                        sesion.FechaFin = edicionGrid.FechaFin;
                        sesion.DuracionMinutos = edicionGrid.DuracionMinutos;
                        sesion.Ubicacion = Common.Constants.STR_ALTA_NUEVA_SESION_NO_PRESENCIAL;
                        sesion.Edicion = edicionGrid;
                        edicionGrid.Sesiones.Add(sesion);
                    }
                    else
                    {
                        DateTime dtReferencia = edicionGrid.FechaInicio.AddDays(-1);
                        for (int intContador = 0; intContador < edicionGrid.SesionesDias; intContador++)
                        {
                            sesion = new Sesion();
                            sesion.Numero = intContador + 1;
                            sesion.DuracionMinutos = (int)edicionGrid.DuracionMinutos / edicionGrid.SesionesDias;

                            if (dtReferencia < edicionGrid.FechaFin) dtReferencia = dtReferencia.AddDays(1);

                            while (dtReferencia.DayOfWeek == DayOfWeek.Saturday || dtReferencia.DayOfWeek == DayOfWeek.Sunday)
                            {
                                dtReferencia = dtReferencia.AddDays(1);
                            }

                            sesion.FechaInicio = dtReferencia.AddHours(8);
                            sesion.FechaFin = dtReferencia.AddHours(8).AddMinutes(sesion.DuracionMinutos);

                            sesion.Ubicacion = Common.Constants.STR_ALTA_NUEVA_SESION_PRESENCIAL;
                            sesion.Edicion = edicionGrid;
                            edicionGrid.Sesiones.Add(sesion);
                        }
                        edicionGrid.FechaFin = dtReferencia;
                    }
                    EdicionService.Insertar(edicionGrid);
                    break;
                case "edit":
                    Edicion edicion = EdicionService.Obtener(Int32.Parse(formCollection["id"]));
                    edicion.FechaInicio = edicionGrid.FechaInicio;
                    edicion.FechaFin = edicionGrid.FechaFin;
                    edicion.Version = edicionGrid.Version;
                    EdicionService.Actualizar(edicion);
                    break;
                case "del":
                    edicion = EdicionService.Obtener(Int32.Parse(formCollection["id"]));
                    EdicionService.Eliminar(edicion);
                    break;
            }
            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }

        /// <summary>
        /// Capturador que almacena la hoja de firmas asociada a la edición
        /// </summary>
        /// <param name="FileData">Contenedor con los datos del fichero</param>
        /// <param name="id">Identificador de la edición</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        public string Upload(HttpPostedFileBase FileData, int id)
        {
            LogFactory.Escribir().Info("Inicio");

            if (Request.Files.Count > 0)
            {
                Edicion edicion = EdicionService.Obtener(id);
                byte[] buffer = new byte[Request.Files[0].ContentLength];
                Request.Files[0].InputStream.Read(buffer, 0, Request.Files[0].ContentLength);
                edicion.HojaFirmasFilename = System.IO.Path.GetFileName(Request.Files[0].FileName);
                edicion.HojaFirmas = buffer;
                EdicionService.Actualizar(edicion);
            }

            LogFactory.Escribir().Info("Fin");
            return "Upload OK";
        }

        /// <summary>
        /// Capturador para descargar la hoja de firmas asociada a la edición
        /// </summary>
        /// <param name="id">Identificador de la edición</param>
        /// <returns>Hoja de firmas de la edición</returns>
        public ActionResult Firmas(string id)
        {
            LogFactory.Escribir().Info("Inicio");

            Edicion edicion = EdicionService.Obtener(Int32.Parse(id));

            LogFactory.Escribir().Info("Fin: " + edicion.HojaFirmas == null);
            if (edicion.HojaFirmas == null) return View();
            return File(edicion.HojaFirmas, "application/octet-stream", Utilities.NormalizarCadena(edicion.HojaFirmasFilename));
        }

        /// <summary>
        /// Capturador para eliminar la hoja de firmas asociada a la edición
        /// </summary>
        /// <param name="edicionActual">Identificador de la edición</param>
        public void FirmasEliminar(int edicionActual)
        {
            LogFactory.Escribir().Info("Inicio");

            Edicion edicion = EdicionService.Obtener(edicionActual);
            edicion.HojaFirmasFilename = null;
            edicion.HojaFirmas = null;
            EdicionService.Actualizar(edicion);

            LogFactory.Escribir().Info("Fin");
        }
        #endregion

        #region Sesiones
        /// <summary>
        /// Capturador que obtiene las sesiones asignadas a la edición indicada
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="edicionActual">Identificador de la sesión</param>
        /// <returns>Colección de sesiones asignadas a la edición</returns>
        public ActionResult ObtenerSesiones(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int edicionActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Sesion> datos;
            Sesion sesion = new Sesion();

            if (edicionActual != 0)
            {
                sesion.Edicion.Idint = edicionActual;
                datos = SesionService.ObtenerLista(sesion);
            }
            else
            {
                datos = new List<Sesion>();
            }

            Func<Sesion, IComparable> funcOrden = SesionService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               a.Numero.ToString(),
                               a.FechaInicio.ToString("dd/MM/yyyy HH:mm"),
                               a.FechaFin.ToString("dd/MM/yyyy HH:mm"),
                               a.Ubicacion,
                               a.DuracionMinutos.ToString()
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar las sesiones
        /// </summary>
        /// <param name="sesionGrid">Datos de la sesión introducida por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarSesion(Sesion sesionGrid, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");

            switch (formCollection["oper"])
            {
                case "edit":
                    Sesion sesion = new Sesion();
                    sesion = SesionService.Obtener(Int32.Parse(formCollection["id"]));
                    TimeSpan tsAnterior = sesion.FechaFin - sesion.FechaInicio;
                    sesion.FechaInicio = DateTime.Parse(formCollection["FInicio"]);
                    sesion.FechaFin = DateTime.Parse(formCollection["FFin"]);
                    sesion.Ubicacion = sesionGrid.Ubicacion;
                    if (sesion.Edicion.Version.EqualTipo(VersionesTipo.Presencial))
                    {
                        TimeSpan tsActual = sesion.FechaFin - sesion.FechaInicio;
                        sesion.DuracionMinutos = (int)tsActual.TotalMinutes;
                        sesion.Edicion.DuracionMinutos += Int32.Parse((tsActual.TotalMinutes - tsAnterior.TotalMinutes).ToString());
                    }
                    SesionService.Actualizar(sesion);
                    EdicionService.RecalcularEdicion(sesion.Edicion);
                    break;
            }

            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }
        #endregion

        #region Convocatorias
        /// <summary>
        /// Capturador que obtiene las convocatorias a la edición indicada
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="edicionActual">Identificador de la edición</param>
        /// <returns>Colección de convocatorias a la edición</returns>
        public ActionResult ObtenerEmpleadosConvocados(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int edicionActual)
        {
            LogFactory.Escribir().Info("Inicio");
            
            IEnumerable<Convocatoria> datos;
            Convocatoria convocatoria = new Convocatoria();
            convocatoria.Edicion = new Edicion();

            if (edicionActual != 0)
            {
                convocatoria.Edicion.Idint = edicionActual;
                datos = ConvocatoriaService.ObtenerLista(convocatoria);
            }
            else
            {
                datos = new List<Convocatoria>();
            }

            Func<Convocatoria, IComparable> funcOrden = ConvocatoriaService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            EmpleadoColectivo ec;
            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Empleado.Idint,
                           cell = new string[] {
                               edicionActual.ToString(),
                               a.Empleado.Apellidos,
                               a.Empleado.Nombre,
                               a.Empleado.Apellidos + ", " + a.Empleado.Nombre,
                               ((ec = EmpleadoColectivoService.ObtenerOcupacion(a.Empleado.Idint)) != null ? ec.Colectivo.Nombre : "?"),
                               (a.Asistencia*100).ToString(),
                               (a.Completado?Constants.STR_VERDADERO:Constants.STR_FALSO),
                               a.Observaciones,
                               (a.Empleado.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador que obtiene los empleados no convocados a la edición indicada
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="edicionActual">Identificador de la edición</param>
        /// <returns>Colección de empleados no convocados a la edición</returns>
        public ActionResult ObtenerEmpleadosNoConvocados(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int edicionActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Empleado> datos;
            Convocatoria convocatoria = new Convocatoria();
            convocatoria.Edicion = new Edicion();

            if (edicionActual != 0)
            {
                convocatoria.Edicion.Idint = edicionActual;

                if (_search)
                {
                    Empleado empleado;
                    switch (searchField)
                    {
                        case "Nombre":
                            empleado = new Empleado();
                            empleado.Nombre = searchString;
                            datos = EmpleadoService.ObtenerEmpleadosNoConvocados(empleado, convocatoria.Edicion);
                            break;
                        case "Apellidos":
                            empleado = new Empleado();
                            empleado.Apellidos = searchString;
                            datos = EmpleadoService.ObtenerEmpleadosNoConvocados(empleado, convocatoria.Edicion);
                            break;
                        default:
                            datos = EmpleadoService.ObtenerEmpleadosNoConvocados(null, convocatoria.Edicion);
                            break;
                    }
                }
                else
                {
                    datos = EmpleadoService.ObtenerEmpleadosNoConvocados(null, convocatoria.Edicion);
                }
            }
            else
            {
                datos = new List<Empleado>();
            }

            Func<Empleado, IComparable> funcOrden = EmpleadoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            EmpleadoColectivo ec;
            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               null,
                               a.Apellidos,
                               a.Nombre,
                               a.Apellidos + ", " + a.Nombre,
                               ((ec = EmpleadoColectivoService.ObtenerOcupacion(a.Idint)) != null ? ec.Colectivo.Nombre : "?"),
                               ((ec != null) && (ec.FechaAsignacion!=DateTime.MinValue) ? ec.FechaAsignacion.ToShortDateString() : "?"),
                               (a.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Desasignación del empleado a la edición
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="edicionIdint">Identificador de la edición</param>
        public void DesconvocarEmpleado(int empleadoIdint, int edicionIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            
            Convocatoria convocatoria = ConvocatoriaService.Obtener(edicionIdint, empleadoIdint);
            ConvocatoriaService.Eliminar(convocatoria);

            Edicion edicion = EdicionService.Obtener(edicionIdint);
            Asistencia asistencia;
            foreach (Sesion sesion in edicion.Sesiones)
            {
                asistencia = AsistenciaService.Obtener(sesion.Idint, empleadoIdint);
                if (asistencia != null) AsistenciaService.Eliminar(asistencia);
            }

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Asignación del empleado a la edición
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="edicionIdint">Identificador de la edición</param>
        public void ConvocarEmpleado(int empleadoIdint, int edicionIdint)
        {
            LogFactory.Escribir().Info("Inicio (" + empleadoIdint + "," + edicionIdint + ")");
            
            Empleado empleado = EmpleadoService.Obtener(empleadoIdint);
            Edicion edicion = EdicionService.Obtener(edicionIdint);

            Convocatoria convocatoria = new Convocatoria();
            convocatoria.Empleado = empleado;
            convocatoria.Edicion = edicion;
            convocatoria.Completado = false;
            ConvocatoriaService.Insertar(convocatoria);

            Asistencia asistencia;
            foreach (Sesion sesion in edicion.Sesiones)
            {
                asistencia = new Asistencia();
                asistencia.Empleado = empleado;
                asistencia.Sesion = sesion;
                AsistenciaService.Insertar(asistencia);
            }
            
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Modificación de la convocatoria del empleado a la sesión
        /// </summary>
        /// <param name="formCollection">Colección de datos indicados por el usuario</param>
        public void EditarConvocatoria(FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            int intEdicion = int.Parse(formCollection["edicionActual"]);
            int intEmpleado = int.Parse(formCollection["id"]);

            Convocatoria convocatoria = ConvocatoriaService.Obtener(intEdicion, intEmpleado);
            if (formCollection.Get("Observaciones") != null) convocatoria.Observaciones = formCollection["Observaciones"];
            convocatoria.Completado = formCollection["Completado"].Equals(Constants.STR_VERDADERO);

            if (convocatoria.Completado)
            {
                AsistenciaService.CompletarAsistencia(intEmpleado, intEdicion, false);
                ConvocatoriaService.RecalcularAsistencia(intEmpleado, intEdicion, false);
            }

            ConvocatoriaService.Actualizar(convocatoria);

            LogFactory.Escribir().Info("Fin");
        }
        #endregion

        #region Control de Asistencia
        /// <summary>
        /// Capturador que obtiene la asistencia de los empleados a la sesión indicada
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="sesionActual">Identificador de la sesión</param>
        /// <returns>Colección de asistencia de los empleados a la sesión</returns>
        public ActionResult ObtenerAsistencias(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int sesionActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Asistencia> datos;
            Asistencia asistencia = new Asistencia();

            if (sesionActual != 0)
            {
                asistencia.Sesion = SesionService.Obtener(sesionActual);
                datos = AsistenciaService.ObtenerLista(asistencia);
            }
            else
            {
                datos = new List<Asistencia>();
            }

            Func<Asistencia, IComparable> funcOrden = AsistenciaService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            EmpleadoColectivo ec;
            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Empleado.Idint,
                           cell = new string[] {
                               a.Empleado.Apellidos,
                               a.Empleado.Nombre,
                               a.Empleado.Apellidos + ", " + a.Empleado.Nombre,
                               ((ec = EmpleadoColectivoService.ObtenerOcupacion(a.Empleado.Idint)) != null ? ec.Colectivo.Nombre : "?"),
                               (a.Asiste?Constants.STR_VERDADERO:Constants.STR_FALSO),
                               (a.Empleado.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Modificación de la asistencia del empleado a la sesión
        /// </summary>
        /// <param name="formCollection">Colección de datos indicados por el usuario</param>
        public void AsignarAsistencia(FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            int intSesion = int.Parse(formCollection["sesionActual"]);
            int intEmpleado = int.Parse(formCollection["id"]);

            Asistencia asistencia = AsistenciaService.Obtener(intSesion, intEmpleado);
            asistencia.Asiste = formCollection["Asiste"].Equals(Constants.STR_VERDADERO);

            AsistenciaService.Actualizar(asistencia);

            Sesion sesion = SesionService.Obtener(intSesion);
            IList<Asistencia> lstAsistencia = AsistenciaService.ObtenerLista(intEmpleado, sesion.Edicion.Idint);

            ConvocatoriaService.RecalcularAsistencia(intEmpleado, sesion.Edicion.Idint, true);

            LogFactory.Escribir().Info("Fin");
        }
        #endregion
    }
}
