﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web.Mvc;
using Common.Log;
using EntornoWeb.Common;
using NHibernate_SGF.Entities;
using NHibernate_SGF.Services;

namespace EntornoWeb.Controllers
{
    /// <summary>
    /// Controlador del mantenimiento de empleados e itinerarios
    /// </summary>
    [HandleError]
    public class MEmpleadosController : SGFController
    {

        #region Empleados
        /// <summary>
        /// Capturador que obtiene los empleados que aparecerán en la tabla
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <returns>Colección con los empleados solicitados</returns>
        public ActionResult ObtenerDatosGrid(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Empleado> datos;
            Empleado empleado;
            switch (searchField)
            {
                case "DNI":
                    empleado = new Empleado();
                    empleado.DNI = searchString;
                    datos = EmpleadoService.ObtenerLista(empleado);
                    break;
                case "Nombre":
                    empleado = new Empleado();
                    empleado.Nombre = searchString;
                    datos = EmpleadoService.ObtenerLista(empleado);
                    break;
                case "Apellidos":
                    empleado = new Empleado();
                    empleado.Apellidos = searchString;
                    datos = EmpleadoService.ObtenerLista(empleado);
                    break;
                default:
                    datos = EmpleadoService.ObtenerLista();
                    break;
            }

            Func<Empleado, IComparable> funcOrden = EmpleadoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            EmpleadoColectivo ec;
            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               a.Idint.ToString(),
                               a.DNI,
                               a.Nombre,
                               a.Apellidos,
                               a.Apellidos + ", " + a.Nombre,
                               (a.Mail != null ? a.Mail : ""),
                               (a.FechaIngreso != DateTime.MinValue ? a.FechaIngreso.ToShortDateString(): ""),
                               (a.FechaBaja != DateTime.MinValue ? a.FechaBaja.ToShortDateString(): ""),
                               ((ec = EmpleadoColectivoService.ObtenerOcupacion(a.Idint)) != null ? ec.Colectivo.Nombre : MensajeUsuario.STR_NO_ASIGNADO),
                               ((ec != null) && (ec.FechaAsignacion!=DateTime.MinValue) ? ec.FechaAsignacion.ToShortDateString() : ""),
                               (a.FechaBaja != DateTime.MinValue ? Constants.STR_VERDADERO : Constants.STR_FALSO)
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador para añadir, modificar y eliminar los empleados
        /// </summary>
        /// <param name="empleado">Datos del empleado introducido por el usuario</param>
        /// <param name="formCollection">Colección con los datos de la acción del usuario</param>
        /// <returns>Resultado de la acción (OK/Excepción)</returns>
        [HttpPost]
        public ActionResult EditarDatosGrid(Empleado empleado, FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            empleado.FechaModi = DateTime.Now;
            switch (formCollection["oper"])
            {
                case "add":
                    EmpleadoService.Insertar(empleado);
                    break;
                case "edit":
                    empleado.Idint = Int32.Parse(formCollection["id"]);
                    EmpleadoService.Actualizar(empleado);
                    if (empleado.FechaBaja != DateTime.MinValue)
                    {
                        empleado = EmpleadoService.Obtener(Int32.Parse(formCollection["id"]));
                        EmpleadoService.EliminarConfiguracionEmpleado(empleado);
                    }
                    break;
                case "del":
                    empleado = EmpleadoService.Obtener(Int32.Parse(formCollection["id"]));
                    if (empleado.FechaBaja == DateTime.MinValue)
                    {
                        empleado.FechaModi = DateTime.Now;
                        empleado.FechaBaja = DateTime.Now;
                        EmpleadoService.Actualizar(empleado);
                        EmpleadoService.EliminarConfiguracionEmpleado(empleado);
                    }
                    break;
            }
            LogFactory.Escribir().Info("Fin");
            return Json("OK");
        }
        #endregion

        #region Cursos
        /// <summary>
        /// Capturador que obtiene los cursos asignados al empleado indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="empleadoActual">Identificador del empleado</param>
        /// <returns>Colección de cursos asignados al empleado</returns>
        public ActionResult ObtenerCursosAsignados(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int empleadoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<EmpleadoCurso> datos;
            EmpleadoCurso empleadoCurso = new EmpleadoCurso();
            empleadoCurso.Empleado = new Empleado();
            empleadoCurso.Curso = new Curso();

            if (empleadoActual != 0)
            {
                empleadoCurso.Empleado.Idint = empleadoActual;
                if (_search)
                {
                    switch (searchField)
                    {
                        case "Nombre":
                            empleadoCurso.Curso.Nombre = searchString;
                            datos = EmpleadoCursoService.ObtenerLista(empleadoCurso);
                            break;
                        case "Tipo":
                            empleadoCurso.Curso.Tipo = searchString;
                            datos = EmpleadoCursoService.ObtenerLista(empleadoCurso);
                            break;
                        default:
                            datos = EmpleadoCursoService.ObtenerLista(empleadoCurso);
                            break;
                    }
                }
                else
                {
                    datos = EmpleadoCursoService.ObtenerLista(empleadoCurso);
                }
            }
            else
            {
                datos = new List<EmpleadoCurso>();
            }

            Func<EmpleadoCurso, IComparable> funcOrden = EmpleadoCursoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Curso.Idint,
                           cell = new string[] {
                               a.Empleado.Idint.ToString(),
                               a.Curso.Nombre,
                               a.Curso.ToString(),
                               a.Exento.ToString(),
                               a.FechaAsignacion.ToShortDateString()
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Capturador que obtiene los cursos no asignados al empleado indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual se ordenan los registros solicitados</param>
        /// <param name="sord">Ordenación del listado (asc/des)</param>
        /// <param name="page">Página solicitada</param>
        /// <param name="rows">Número de registros solicitados</param>
        /// <param name="_search">Se reciben parámetros de filtrado</param>
        /// <param name="searchField">Campo por el cual se filtra</param>
        /// <param name="searchOper">Operador del filtro</param>
        /// <param name="searchString">Cadena filtrada</param>
        /// <param name="empleadoActual">Identificador del empleado</param>
        /// <returns>Colección de cursos no asignados al empleado</returns>
        public ActionResult ObtenerCursosNoAsignados(string sidx, string sord, int page, int rows, bool _search, string searchField, string searchOper, string searchString, int empleadoActual)
        {
            LogFactory.Escribir().Info("Inicio");

            IEnumerable<Curso> datos;
            EmpleadoCurso empleadoCurso = new EmpleadoCurso();
            empleadoCurso.Empleado = new Empleado();
            empleadoCurso.Curso = new Curso();

            if (empleadoActual != 0)
            {
                empleadoCurso.Empleado.Idint = empleadoActual;

                if (_search)
                {
                    switch (searchField)
                    {
                        case "Nombre":
                            empleadoCurso.Curso.Nombre = searchString;
                            datos = CursoService.ObtenerListaNoAsignadoEmpleado(empleadoCurso);
                            break;
                        case "Tipo":
                            empleadoCurso.Curso.Tipo = searchString;
                            datos = CursoService.ObtenerListaNoAsignadoEmpleado(empleadoCurso);
                            break;
                        default:
                            datos = CursoService.ObtenerListaNoAsignadoEmpleado(empleadoCurso);
                            break;
                    }
                }
                else
                {
                    datos = CursoService.ObtenerListaNoAsignadoEmpleado(empleadoCurso);
                }
            }
            else
            {
                datos = new List<Curso>();
            }

            Func<Curso, IComparable> funcOrden = CursoService.CrearFuncionOrdenacion(sidx);
            if (sord == Constants.STR_ORDENACION_ASCENDENTE)
            {
                datos = datos.OrderBy(funcOrden);
            }
            else
            {
                datos = datos.OrderByDescending(funcOrden);
            }

            int totalRegistros = datos.Count();
            int totalPages = (int)Math.Ceiling((decimal)totalRegistros / (decimal)rows);
            datos = datos.Skip((page - 1) * rows).Take(rows);

            var data = new
            {
                total = totalPages,
                page = page,
                records = totalRegistros,
                rows = from a in datos
                       select new
                       {
                           id = a.Idint,
                           cell = new string[] {
                               null,
                               a.Nombre,
                               a.ToString()
                           }
                       }
            };
            LogFactory.Escribir().Info("Fin");
            return Json(data);
        }

        /// <summary>
        /// Desasignación del curso al empleado
        /// </summary>
        /// <param name="cursoIdint">Identificador del curso</param>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        public void DesasignarCurso(int cursoIdint, int empleadoIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            Curso curso = CursoService.Obtener(cursoIdint);
            Empleado empleado = EmpleadoService.Obtener(empleadoIdint);

            EmpleadoCurso empleadoCurso = new EmpleadoCurso();
            empleadoCurso.Curso = curso;
            empleadoCurso.Empleado = empleado;

            EmpleadoCursoService.Eliminar(empleadoCurso);
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Asignación del curso al empleado
        /// </summary>
        /// <param name="cursoIdint">Identificador del curso</param>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        public void AsignarCurso(int cursoIdint, int empleadoIdint)
        {
            LogFactory.Escribir().Info("Inicio");
            Curso curso = CursoService.Obtener(cursoIdint);
            Empleado empleado = EmpleadoService.Obtener(empleadoIdint);

            EmpleadoColectivo empleadoColectivo = EmpleadoColectivoService.ObtenerOcupacion(empleadoIdint);

            EmpleadoCurso empleadoCurso = new EmpleadoCurso();
            empleadoCurso.Curso = curso;
            empleadoCurso.Empleado = empleado;
            empleadoCurso.Exento = false;
            if (empleadoColectivo != null && empleadoColectivo.FechaAsignacion != DateTime.MinValue)
            {
                empleadoCurso.FechaAsignacion = empleadoColectivo.FechaAsignacion;
            }
            else
            {
                empleadoCurso.FechaAsignacion = DateTime.Now;
            }

            EmpleadoCursoService.Insertar(empleadoCurso);
            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Modificación de la asignación del curso al empleado
        /// </summary>
        /// <param name="formCollection">Colección de datos indicados por el usuario</param>
        public void EditarAsignacion(FormCollection formCollection)
        {
            LogFactory.Escribir().Info("Inicio");
            int intCurso = int.Parse(formCollection["id"]);
            int intEmpleado = int.Parse(formCollection["EmpleadoIdint"]);

            EmpleadoCurso empleadoCurso = EmpleadoCursoService.Obtener(intEmpleado, intCurso);
            empleadoCurso.FechaAsignacion = DateTime.Parse(formCollection["FechaAsignacion"]);
            empleadoCurso.Exento = Boolean.Parse(formCollection["Exento"]);

            EmpleadoCursoService.Actualizar(empleadoCurso);
            LogFactory.Escribir().Info("Fin");
        }
        #endregion

    }
}
