﻿using System;
using System.Collections.Generic;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de asistencias
    /// </summary>
    public class AsistenciaService : MasterService
    {
        /// <summary>
        /// Obtiene la asistencia que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intSesionIdint">Identificador de la sesión</param>
        /// <param name="p_intEmpleadoIdint">Identificador del empleado</param>
        /// <returns>Asistencia</returns>
        public static Asistencia Obtener(int p_intSesionIdint, int p_intEmpleadoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Asistencia>();
            query.CreateCriteria("Sesion", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intSesionIdint));
            query.CreateCriteria("Empleado", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEmpleadoIdint));

            Asistencia objeto = query.UniqueResult<Asistencia>();

            LogFactory.Escribir().Info("Fin? " + (objeto != null));
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las asistencias
        /// </summary>
        /// <returns>Lista de asistencias</returns>
        public static IList<Asistencia> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Asistencia>();
            IList<Asistencia> lstObjetos = query.List<Asistencia>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de asistencias que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="asistencia">Parámetros de filtrado de las asistencias</param>
        /// <returns>Lista de asistencias</returns>
        public static IList<Asistencia> ObtenerLista(Asistencia asistencia)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Asistencia>();
            if (asistencia.Sesion != null && asistencia.Sesion.Idint != 0) query.Add(Restrictions.Eq("Sesion.Idint", asistencia.Sesion.Idint));
            if (asistencia.Empleado != null && asistencia.Empleado.Idint != 0) query.Add(Restrictions.Eq("Empleado.Idint", asistencia.Empleado.Idint));
            ICriteria queryA = query.CreateCriteria("Sesion", JoinType.LeftOuterJoin);
            if (asistencia.Sesion != null && asistencia.Sesion.Numero != 0) queryA.Add(Restrictions.Eq("Numero", asistencia.Sesion.Numero));
            ICriteria queryB = query.CreateCriteria("Empleado", JoinType.LeftOuterJoin);
            if (asistencia.Empleado != null && asistencia.Empleado.Nombre != null) queryB.Add(Restrictions.Like("Nombre", asistencia.Empleado.Nombre, MatchMode.Anywhere));

            IList<Asistencia> lstObjetos = query.List<Asistencia>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de asistencias que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="edicionIdint">Identificador de la edición</param>
        /// <returns>Lista de asistencias</returns>
        public static IList<Asistencia> ObtenerLista(int empleadoIdint, int edicionIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Asistencia>();
            query.Add(Restrictions.Eq("Empleado.Idint", empleadoIdint));
            query = query.CreateCriteria("Sesion", JoinType.InnerJoin);
            query = query.CreateCriteria("Edicion", JoinType.InnerJoin);
            query.Add(Restrictions.Eq("Idint", edicionIdint));

            IList<Asistencia> lstObjetos = query.List<Asistencia>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Asigna el estado de la asistencia del empleado a todas las sesiones de la edición
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="edicionIdint">Identificador de la edición</param>
        /// <param name="blnCommit">Estado de asistencia (Asiste/No asiste)</param>
        public static void CompletarAsistencia(int empleadoIdint, int edicionIdint, bool blnCommit)
        {
            LogFactory.Escribir().Info("Inicio");

            foreach (Asistencia asistencia in AsistenciaService.ObtenerLista(empleadoIdint, edicionIdint))
            {
                asistencia.Asiste = true;
                if (blnCommit) AsistenciaService.Actualizar(asistencia);
            }

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Ordena una lista de asistencias en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las asistencias</returns>
        public static Func<Asistencia, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Asistencia, IComparable> funcOrden =
                sidx == "Empleado" ? a => a.Empleado.Apellidos :
                sidx == "Apellidos" ? a => a.Empleado.Apellidos :
                sidx == "Nombre" ? a => a.Empleado.Nombre :
                sidx == "Asiste" ? a => a.Asiste :
                (Func<Asistencia, IComparable>)(a => a.Sesion.FechaInicio);
            return funcOrden;
        }
    }
}
