﻿using System;
using System.Collections.Generic;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de convocatorias
    /// </summary>
    public class ConvocatoriaService : MasterService
    {
        /// <summary>
        /// Obtiene la convocatoria que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intEdicioIdint">Identificador de la edición</param>
        /// <param name="p_intEmpleadoIdint">Identificador del empleado</param>
        /// <returns>Convocatoria</returns>
        public static Convocatoria Obtener(int p_intEdicioIdint, int p_intEmpleadoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Convocatoria>();
            query.CreateCriteria("Edicion", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEdicioIdint));
            query.CreateCriteria("Empleado", JoinType.FullJoin).Add(Restrictions.Eq("Idint", p_intEmpleadoIdint));

            Convocatoria objeto = query.UniqueResult<Convocatoria>();

            LogFactory.Escribir().Info("Fin? " + (objeto != null));
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las convocatorias
        /// </summary>
        /// <returns>Lista de convocatorias</returns>
        public static IList<Convocatoria> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Convocatoria>();
            IList<Convocatoria> lstObjetos = query.List<Convocatoria>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de convocatorias que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="convocatoria">Parámetros de filtrado de las convocatorias</param>
        /// <returns>Lista de convocatorias</returns>
        public static IList<Convocatoria> ObtenerLista(Convocatoria convocatoria)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Convocatoria>();
            if (convocatoria.Edicion != null && convocatoria.Edicion.Idint != 0) query.Add(Restrictions.Eq("Edicion.Idint", convocatoria.Edicion.Idint));
            if (convocatoria.Empleado != null && convocatoria.Empleado.Idint != 0) query.Add(Restrictions.Eq("Empleado.Idint", convocatoria.Empleado.Idint));
            ICriteria queryA = query.CreateCriteria("Edicion", JoinType.LeftOuterJoin);
            if (convocatoria.Edicion != null && convocatoria.Edicion.FechaInicio != DateTime.MinValue) queryA.Add(Restrictions.Eq("FechaInicio", convocatoria.Edicion.FechaInicio));
            if (convocatoria.Edicion != null && convocatoria.Edicion.FechaFin != DateTime.MinValue) queryA.Add(Restrictions.Eq("FechaFin", convocatoria.Edicion.FechaInicio));
            ICriteria queryB = query.CreateCriteria("Empleado", JoinType.LeftOuterJoin);
            if (convocatoria.Empleado != null && convocatoria.Empleado.Nombre != null) queryB.Add(Restrictions.Like("Nombre", convocatoria.Empleado.Nombre, MatchMode.Anywhere));
            
            IList<Convocatoria> lstObjetos = query.List<Convocatoria>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de convocatorias que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado del empleado de las convocatorias</param>
        /// <param name="curso">Parámetros de filtrado del curso de las convocatorias</param>
        /// <returns>Lista de convocatorias</returns>
        public static IList<Convocatoria> ObtenerLista(Empleado empleado, Curso curso)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Convocatoria>("cv");
            if (empleado.Idint != 0) query.Add(Restrictions.Eq("Empleado.Idint", empleado.Idint));

            query = query.CreateCriteria("Edicion", "ed", JoinType.LeftOuterJoin);
            query = query.CreateCriteria("Version", "ve", JoinType.LeftOuterJoin);
            query = query.CreateCriteria("Curso", "cu", JoinType.LeftOuterJoin);

            ICriterion cursoRepetirNuevaVersion = Restrictions.Eq("cu.RepetirConNuevaVersion", true);
            ICriterion cursoNoRepetirNuevaVersion = Restrictions.Eq("cu.RepetirConNuevaVersion", false);
            ICriterion versionSinFechaBaja = Restrictions.IsNotNull("ve.FechaBaja");
                
            Conjunction conjuncion = Restrictions.Conjunction();
            conjuncion.Add(cursoRepetirNuevaVersion);
            conjuncion.Add(versionSinFechaBaja);

            Disjunction disjuncion = Restrictions.Disjunction();
            disjuncion.Add(cursoNoRepetirNuevaVersion);
            disjuncion.Add(conjuncion);

            if (curso.Idint != 0) query.Add(Restrictions.Eq("cu.Idint", curso.Idint));
            query.Add(disjuncion);
            query.AddOrder(Order.Desc("cv.Completado"));

            IList<Convocatoria> lstObjetos = query.List<Convocatoria>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Calcula la asistencia media del empleado a la edición
        /// </summary>
        /// <param name="empleadoIdint">Identificador del empleado</param>
        /// <param name="edicionIdint">Identificador de la edición</param>
        /// <param name="blnCommit">Almacenar el valor obtenido</param>
        public static void RecalcularAsistencia(int empleadoIdint, int edicionIdint, bool blnCommit)
        {
            LogFactory.Escribir().Info("Inicio");

            Convocatoria convocatoria = ConvocatoriaService.Obtener(edicionIdint, empleadoIdint);
            
            IList<Asistencia> lstAsistencias = AsistenciaService.ObtenerLista(empleadoIdint, edicionIdint);
            int intContador = 0;
            foreach (Asistencia asistencia in lstAsistencias)
            {
                if (asistencia.Asiste) intContador++;
            }

            convocatoria.Asistencia = (float)intContador / (float) lstAsistencias.Count;
            if (blnCommit) ConvocatoriaService.Actualizar(convocatoria);

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Ordena una lista de convocatorias en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las convocatorias</returns>
        public static Func<Convocatoria, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Convocatoria, IComparable> funcOrden =
                sidx == "FechaInicio" ? a => a.Edicion.FechaInicio :
                sidx == "FechaFin" ? a => a.Edicion.FechaFin :
                sidx == "Nombre" ? a => a.Empleado.Nombre :
                sidx == "Apellidos" ? a => a.Empleado.Apellidos :
                sidx == "Empleado" ? a => a.Empleado.Apellidos :
                sidx == "Asistencia" ? a => a.Asistencia :
                sidx == "Completado" ? a => a.Completado :
                (Func<Convocatoria, IComparable>)(a => a.Empleado.Apellidos);
            return funcOrden;
        }
    }
}
