﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de cursos
    /// </summary>
    public class CursoService : MasterService
    {
        /// <summary>
        /// Obtiene el curso que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador del curso</param>
        /// <returns>Curso</returns>
        public static Curso Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Curso>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Curso curso = query.UniqueResult<Curso>();

            LogFactory.Escribir().Info("Fin");
            return curso;
        }

        /// <summary>
        /// Obtiene una lista con todos los cursos que no están dados de baja
        /// </summary>
        /// <returns>Lista de cursos</returns>
        public static IList<Curso> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Curso>();
            query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Curso> lstObjetos = query.List<Curso>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }
        
        /// <summary>
        /// Obtiene una lista de cursos que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="curso">Parámetros de filtrado de los cursos</param>
        /// <returns>Lista de cursos</returns>
        public static IList<Curso> ObtenerLista(Curso curso)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Curso>();
            if (curso != null && curso.Idint != 0) query.Add(Restrictions.Eq("Idint", curso.Idint));
            if (curso != null && curso.Nombre != null) query.Add(Restrictions.Like("Nombre", curso.Nombre, MatchMode.Anywhere));
            if (curso != null && curso.Descripcion != null) query.Add(Restrictions.Like("Descripcion", curso.Descripcion, MatchMode.Anywhere));
            if (curso != null && curso.Tipo != null) query.Add(Restrictions.Eq("Tipo", Curso.CorrespondenciaTipo(curso.Tipo)));
            if (curso != null && curso.PeriodicidadMeses != 0) query.Add(Restrictions.Eq("PeriodicidadMeses", curso.PeriodicidadMeses));
            if (curso != null && curso.IncorporacionMeses != 0) query.Add(Restrictions.Eq("IncorporacionMeses", curso.IncorporacionMeses));
            query.Add(Restrictions.IsNull("FechaBaja"));

            IList<Curso> lstObjetos = query.List<Curso>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de cursos que no se encuentran vinculados al empleado indicado
        /// </summary>
        /// <param name="empleadoCurso">Parámetros de filtrado de la vinculación del empleado y el curso</param>
        /// <returns>Lista de cursos</returns>
        public static IList<Curso> ObtenerListaNoAsignadoEmpleado(EmpleadoCurso empleadoCurso)
        {
            LogFactory.Escribir().Info("Inicio");

            DetachedCriteria subquery = DetachedCriteria.For<Curso>("c1");
            subquery.SetProjection(Projections.Property("c1.Idint"));
            subquery.CreateCriteria("EmpleadosCursos", "ec", JoinType.InnerJoin).Add(Restrictions.Eq("Empleado.Idint", empleadoCurso.Empleado.Idint));

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Curso>("c2");
            query.Add(Subqueries.PropertyNotIn("c2.Idint", subquery));
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Idint != 0) query.Add(Restrictions.Eq("Idint", empleadoCurso.Curso.Idint));
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Nombre != null) query.Add(Restrictions.Like("Nombre", empleadoCurso.Curso.Nombre, MatchMode.Anywhere));
            if (empleadoCurso.Curso != null && empleadoCurso.Curso.Tipo != null) query.Add(Restrictions.Eq("Tipo", Curso.CorrespondenciaTipo(empleadoCurso.Curso.Tipo)));
            query.Add(Restrictions.IsNull("FechaBaja"));

            IList<Curso> lstObjetos = query.List<Curso>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Ordena una lista de cursos en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de los cursos</returns>
        public static Func<Curso, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Curso, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "Nombre" ? a => a.Nombre :
                sidx == "Descripcion" ? a => a.Descripcion :
                sidx == "Tipo" ? a => a.Tipo :
                sidx == "PeriodicidadMeses" ? a => a.PeriodicidadMeses :
                sidx == "IncorporacionMeses" ? a => a.IncorporacionMeses :
                sidx == "RepetirConNuevaVersion" ? a => a.RepetirConNuevaVersion :
                (Func<Curso, IComparable>)(a => a.Nombre);
            return funcOrden;
        }
    }
}
