﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de empleados
    /// </summary>
    public class EmpleadoService : MasterService
    {
        /// <summary>
        /// Obtiene el empleado que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador del empleado</param>
        /// <returns>Empleado</returns>
        public static Empleado Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Empleado empleado = query.UniqueResult<Empleado>();

            LogFactory.Escribir().Info("Fin");
            return empleado;
        }

        /// <summary>
        /// Obtiene una lista con todos los empleados
        /// </summary>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>();
            //query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Empleado> lstObjetos = query.List<Empleado>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de empleados que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado de los empleados</param>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerLista(Empleado empleado)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>();
            if (empleado.Idint != 0) query.Add(Restrictions.Eq("Idint", empleado.Idint));
            if (empleado.DNI != null) query.Add(Restrictions.Like("DNI", empleado.DNI, MatchMode.Anywhere));
            if (empleado.Nombre != null) query.Add(Restrictions.Like("Nombre", empleado.Nombre, MatchMode.Anywhere));
            if (empleado.Apellidos != null) query.Add(Restrictions.Like("Apellidos", empleado.Apellidos, MatchMode.Anywhere));
            //query.Add(Restrictions.IsNull("FechaBaja"));
            IList<Empleado> lstObjetos = query.List<Empleado>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de empleados que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado de los empleados</param>
        /// <param name="colectivo">Parámetros de filtrado de los colectivos</param>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerLista(Empleado empleado, Colectivo colectivo) {
            return ObtenerLista(empleado, colectivo, DateTime.MinValue);
        }

        /// <summary>
        /// Obtiene una lista de empleados que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado de los empleados</param>
        /// <param name="colectivo">Parámetros de filtrado del colectivo del empleado</param>
        /// <param name="fechaMinimaAsignacion">Fecha mínima de la asignación del empleado con el colectivo</param>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerLista(Empleado empleado, Colectivo colectivo, DateTime fechaMinimaAsignacion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>();
            if (empleado.Idint != 0) query.Add(Restrictions.Eq("Idint", empleado.Idint));
            if (empleado.DNI != null) query.Add(Restrictions.Like("DNI", empleado.DNI, MatchMode.Anywhere));
            if (empleado.Nombre != null) query.Add(Restrictions.Like("Nombre", empleado.Nombre, MatchMode.Anywhere));
            if (empleado.Apellidos != null) query.Add(Restrictions.Like("Apellidos", empleado.Apellidos, MatchMode.Anywhere));
            //query.Add(Restrictions.IsNull("FechaBaja"));
            query = query.CreateCriteria("EmpleadosColectivos", JoinType.InnerJoin);
            if (fechaMinimaAsignacion != DateTime.MinValue) query.Add(Restrictions.Ge("FechaAsignacion", fechaMinimaAsignacion));
            query = query.CreateCriteria("Colectivo", JoinType.InnerJoin);
            query.Add(Restrictions.Eq("Idint", colectivo.Idint));
            query.Add(Restrictions.IsNull("FechaBaja"));

            IList<Empleado> lstObjetos = query.List<Empleado>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de empleados que coinciden con los parámetros indicados y no se encuentran vinculados al colectivo indicado
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado de los empleados</param>
        /// <param name="colectivo">Parámetros de filtrado del colectivo al que no debe pertenecer el empleado</param>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerListaDistintoColectivo(Empleado empleado, Colectivo colectivo)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>();
            if (empleado.Idint != 0) query.Add(Restrictions.Eq("Idint", empleado.Idint));
            if (empleado.DNI != null) query.Add(Restrictions.Like("DNI", empleado.DNI, MatchMode.Anywhere));
            if (empleado.Nombre != null) query.Add(Restrictions.Like("Nombre", empleado.Nombre, MatchMode.Anywhere));
            if (empleado.Apellidos != null) query.Add(Restrictions.Like("Apellidos", empleado.Apellidos, MatchMode.Anywhere));
            //query.Add(Restrictions.IsNull("FechaBaja"));

            DetachedCriteria subquery = DetachedCriteria.For<EmpleadoColectivo>();
            subquery.SetProjection(Projections.Distinct(Projections.Property("Empleado")));
            subquery.Add(Restrictions.Eq("Colectivo.Idint", colectivo.Idint));

            query.Add(Subqueries.PropertyNotIn("Idint", subquery));

            IList<Empleado> lstObjetos = query.List<Empleado>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de empleados que coinciden con los parámetros indicados y no se encuentran convocados a la edición indicada
        /// </summary>
        /// <param name="empleado">Parámetros de filtrado de los empleados</param>
        /// <param name="edicion">Parámetros de filtrado de la edición a la que no debe estar convocado el empleado</param>
        /// <returns>Lista de empleados</returns>
        public static IList<Empleado> ObtenerEmpleadosNoConvocados(Empleado empleado, Edicion edicion)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Empleado>("empl");
            if (empleado != null && empleado.Idint != 0) query.Add(Restrictions.Eq("Idint", empleado.Idint));
            if (empleado != null && empleado.Nombre != null) query.Add(Restrictions.Like("Nombre", empleado.Nombre, MatchMode.Anywhere));
            if (empleado != null && empleado.Apellidos != null) query.Add(Restrictions.Like("Apellidos", empleado.Apellidos, MatchMode.Anywhere));
            //query.Add(Restrictions.IsNull("FechaBaja"));
            query = query.CreateCriteria("EmpleadosCursos", JoinType.InnerJoin);
            query = query.CreateCriteria("Curso", JoinType.InnerJoin);
            query = query.CreateCriteria("Versiones", JoinType.InnerJoin);
            query = query.CreateCriteria("Ediciones", "edic", JoinType.InnerJoin);
            query.Add(Restrictions.Eq("Idint", edicion.Idint));
            query = query.CreateCriteria("Convocatorias", "conv", JoinType.LeftOuterJoin, Restrictions.EqProperty("Empleado", "empl.Idint"));
            query.Add(
                Restrictions.Or(
                    Restrictions.Conjunction().Add(Restrictions.Eq("Completado", false))
                        .Add(Restrictions.Lt("edic.FechaFin", DateTime.Now))
                        .Add(!Restrictions.Eq("edic.Idint", edicion.Idint)),
                    Restrictions.Conjunction().Add(Restrictions.IsNull("Empleado.Idint"))
                )
            );
            
            IList<Empleado> lstObjetos = query.List<Empleado>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Elimina las vinculaciones del empleado indicado con sus colectivos y cursos
        /// </summary>
        /// <param name="empleado">Empleado al que desvincular</param>
        public static void EliminarConfiguracionEmpleado(Empleado empleado)
        {
            LogFactory.Escribir().Info("Inicio");

            foreach (EmpleadoColectivo empleadoColectivo in empleado.EmpleadosColectivos)
            {
                EmpleadoColectivoService.Eliminar(empleadoColectivo);
            }
            foreach (EmpleadoCurso empleadoCurso in empleado.EmpleadosCursos)
            {
                EmpleadoCursoService.Eliminar(empleadoCurso);
            }

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Ordena una lista de empleados en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de los empleados</returns>
        public static Func<Empleado, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Empleado, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "DNI" ? a => a.DNI :
                sidx == "Nombre" ? a => a.Nombre :
                sidx == "Apellidos" ? a => a.Apellidos :
                sidx == "Empleado" ? a => a.Apellidos :
                sidx == "Mail" ? a => a.Mail :
                sidx == "FechaIngreso" ? a => a.FechaIngreso :
                sidx == "FechaBaja" ? a => a.FechaBaja :
                sidx == "FechaModi" ? a => a.FechaModi :
                (Func<Empleado, IComparable>)(a => a.Apellidos);

            return funcOrden;
        }
    }
}
