﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Common.Log;
using NHibernate;
using NHibernate.Criterion;
using NHibernate.SqlCommand;
using NHibernate_SGF.Entities;

namespace NHibernate_SGF.Services
{
    /// <summary>
    /// Proveedor de acceso a los datos de versiones
    /// </summary>
    public class VersionService : MasterService
    {
        /// <summary>
        /// Obtiene la versión que coincide con los parámetros indicados
        /// </summary>
        /// <param name="p_intIdentificador">Identificador de la versión</param>
        /// <returns>Versión</returns>
        public static Versio Obtener(int p_intIdentificador)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Versio>();
            query.Add(Restrictions.Eq("Idint", p_intIdentificador));
            Versio objeto = query.UniqueResult<Versio>();

            LogFactory.Escribir().Info("Fin");
            return objeto;
        }

        /// <summary>
        /// Obtiene una lista con todas las versiones
        /// </summary>
        /// <returns>Lista de versiones</returns>
        public static IList<Versio> ObtenerLista()
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Versio>();
            IList<Versio> lstObjetos = query.List<Versio>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene una lista de versiones que coinciden con los parámetros indicados
        /// </summary>
        /// <param name="version">Parámetros de filtrado de las versiones</param>
        /// <returns>Lista de versiones</returns>
        public static IList<Versio> ObtenerLista(Versio version)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Versio>();
            if (version.Idint != 0) query.Add(Restrictions.Eq("Idint", version.Idint));
            if (version.Nombre != null) query.Add(Restrictions.Like("Nombre", version.Nombre, MatchMode.Anywhere));
            query = query.CreateCriteria("Curso");
            if (version.Curso != null & version.Curso.Idint != 0) query.Add(Restrictions.Eq("Idint", version.Curso.Idint));
            IList<Versio> lstObjetos = query.List<Versio>();

            LogFactory.Escribir().Info("Fin (#Objetos=" + lstObjetos.Count + ")");
            return lstObjetos;
        }

        /// <summary>
        /// Obtiene la versión activa del curso indicado
        /// </summary>
        /// <param name="p_intCursoIdint">Identificador del curso</param>
        /// <returns>Versión que no tiene fecha de baja</returns>
        public static Versio ObtenerActiva(int p_intCursoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            ICriteria query = SessionManager.Instance.GetSession().CreateCriteria<Versio>();
            query.Add(Restrictions.IsNull("FechaBaja"));
            query = query.CreateCriteria("Curso");
            query.Add(Restrictions.Eq("Idint", p_intCursoIdint));
            Versio objeto = query.UniqueResult<Versio>();

            LogFactory.Escribir().Info("Fin");
            return objeto;
        }

        /// <summary>
        /// Activa la versión más reciente del curso indicado
        /// </summary>
        /// <param name="p_intCursoIdint">Identificador del curso</param>
        public static void ActivarUltima(int p_intCursoIdint)
        {
            LogFactory.Escribir().Info("Inicio");

            String hql = "update Versio set vers_fecha_baja = null, vers_fecha_modi = sysdatetime() where vers_idint = " +
                        "(select cast(MAX(vers_idint) as integer) from Versio " +
                        "where vers_curs_idint = :curso )";
            IQuery query = SessionManager.Instance.GetSession().CreateQuery(hql);
            query.SetInt32("curso", p_intCursoIdint);
            int versiones = query.ExecuteUpdate();

            LogFactory.Escribir().Info("Fin (#Objetos actualizados=" + versiones + ")");
        }

        /// <summary>
        /// Da de baja la versión indicada informando su fecha de baja
        /// </summary>
        /// <param name="version">Versión a dar de baja</param>
        public static void DarDeBaja(Versio version)
        {
            LogFactory.Escribir().Info("Inicio");

            version.FechaModi = DateTime.Now;
            version.FechaBaja = DateTime.Now;
            Actualizar(version);
            ActivarUltima(version.Curso.Idint);

            LogFactory.Escribir().Info("Fin");
        }

        /// <summary>
        /// Da de baja todas las versiones del curso a excepción de la versión indicada
        /// </summary>
        /// <param name="version">Versión que quedará activa</param>
        public static void BajaVersionesRestantes(Versio version)
        {
            LogFactory.Escribir().Info("Inicio");

            String hql = "update Versio set vers_fecha_baja=SYSDATETIME() " +
                            "where vers_fecha_baja is null " +
                                "and vers_curs_idint = :curso " +
                                "and vers_idint <> :version ";
            IQuery query = SessionManager.Instance.GetSession().CreateQuery(hql);
            query.SetInt32("curso", version.Curso.Idint);
            query.SetInt32("version", version.Idint);
            int versiones = query.ExecuteUpdate();

            LogFactory.Escribir().Info("Fin (#Objetos actualizados=" + versiones + ")");
        }

        /// <summary>
        /// Ordena una lista de versiones en base al campo indicado
        /// </summary>
        /// <param name="sidx">Campo por el cual ordena</param>
        /// <returns>Función de ordenación de las versiones</returns>
        public static Func<Versio, IComparable> CrearFuncionOrdenacion(string sidx)
        {
            Func<Versio, IComparable> funcOrden =
                sidx == "Idint" ? a => a.Idint :
                sidx == "Nombre" ? a => a.Nombre :
                sidx == "Tipo" ? a => a.Tipo :
                sidx == "FechaAlta" ? a => a.FechaAlta :
                sidx == "FechaModi" ? a => a.FechaModi :
                sidx == "FechaBaja" ? a => a.FechaBaja :
                (Func<Versio, IComparable>)(a => a.Nombre);
            return funcOrden;
        }
    }
}
